#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Filename: jabber_roster_util.py 
Version: 0.1
Function: export the roster of a jabber account (and imports it to another one).Use '-help' as parameter for further information. 
Date of Last Change: 29.09.2007
    
Copyright (C) 2007 Benjamin Weber <benny@builtin.de> and Patrick Beck <pbeck@yourse.de>  

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import xmpp, os, sys, getpass, time

def connect(jid,password):
    try:
        if not client.connect():
            print 'Could not connect to the server.'
            sys.exit()
        if not client.auth(jid.getNode(),password):
            print 'Could not authenticate with the server.'
            sys.exit()
    except:
        sys.exit()
    print '\nConnection for %s successfully established.' %(jid)
    print '-'*30

def backup(dump):
    try:
        path = file(dump,'w')
        jids = client.getRoster().getItems() # get the contacts
        for contacts in jids:
            name = client.getRoster().getName(contacts) # get the name to a jid
            group = client.getRoster().getGroups(contacts) # get the group to a jid
            contact = str(contacts) + '|' + str(name) + '|' + str(group).replace('[','').replace('u\'','')\
                      .replace('\'','').replace(']','') + '\n' # create a fileformatsyntax
            path.write(contact) # write the data into the given textfile
            print contacts
        path.close()
    except:
        print 'File can not be create. Perhaps the dir not exist or you have not enough rights to access.'

    print '-'*30
    print 'The backup has been written successfully in the file %s \n' %(dump)

def restore(dump):
    try:
        path = file(dump, 'r')
        contactlist = []
        for contacts in path.readlines(): 
            contacts = contacts.split('|')
            contactlist.append(contacts)
        path.close()
    except:
        print 'File / dir not exist or you have not enough rights to access.'
        sys.exit()

    lcontact = len(contactlist)
    i = 0
    while i != lcontact:
        names = contactlist[i][1]
        if contactlist[i][1] == 'None': # if name not set, use a empty string 
            names = ''
        groups = contactlist[i][2].split(',')
        if groups == ['\n'] or groups == ['None\n']:
            groups = ['General']
        infos = client.getRoster().setItem(contactlist[i][0],names.replace('[','').replace(']',''),groups) # sets to the given jid, name and group if exist
        auth = client.getRoster().Authorize(contactlist[i][0])
        subscribe = client.getRoster().Subscribe(contactlist[i][0])
        print contactlist[i][0]
        i += 1 
        time.sleep(1)
    print'-'*30
    print 'Contacts sucessfully added to your roster\n'
    
def version():
    print 'jabber_roster_util.py Version 0.1 by Benjamin Weber <benny@builtin.de> \
and Patrick Beck <pbeck@yourse.de>'
    
def help():
    print '''\nUsage of jabber_roster_util.py 
    
        jabber_roster_util.py is a console programm to backup a 
        jabber account or transfer the data from one to an other account
        
        Syntax: 
        ./jabber_roster_util.py <option> <file for backup or restore> <jid@example.tld>
        
        Options that need the file and jid parameter:
        
            -backup, -b    Backups the given account in file
            -restore, -r    Restore the account with the given file
        
        more options:
        
            -version, -v    Print the current version of this script and the authors
            -help, -h       Print this document
            
        To edit your contacts you can easily make changes directly in the backup/restore file
        in the following syntax:
        
        jid|name|group(s)
        
        For example:
        
        jid@example.com|Jiddy|MyGroup, Jabber, contacts
        
        More Groups for a contact can be separated via a comma
        '''  
        
def main():
    global client # global variable for use in all needed functions
    if len(sys.argv) == 1 or len(sys.argv) == 3 or len(sys.argv) > 4: 
        print 'Use -help for usage information.'
    else:
        if len(sys.argv) == 2:
            if sys.argv[1] == '-help' or sys.argv[1] == '-h':
                help()
                sys.exit()
            if sys.argv[1] == '-version' or sys.argv[1] == '-v':
                version()
                sys.exit()
        if len(sys.argv) == 4:
            dump = sys.argv[2]
            jid = sys.argv[3]
            jid = xmpp.protocol.JID(jid) # create a jid object from the given string
            client = xmpp.Client(jid.getDomain(),debug=[]) # create a client instance
            if sys.argv[1] == '-backup' or sys.argv[1] == '-b':
                try:
                    password = getpass.getpass('Password: ') 
                except:
                    print ''
                    print 'Please enter a password.'
                    sys.exit()
                connect(jid,password)
                backup(dump)
                sys.exit()
            if sys.argv[1] == '-restore' or sys.argv[1] == '-r':
                try:
                    password = getpass.getpass('Password: ')
                except:
                    print ''
                    print 'Please enter a password.'
                    sys.exit()
                connect(jid,password)
                restore(dump)
                sys.exit()
            else:
                print 'You have used an unknown parameter. Use -help for further information.'
        else:
            print 'You have used an unknown parameter. Use -help for further information.'
       
if __name__ == '__main__':
    main()
